@@ -2,11 +2,25 @@ use strict;
 use warnings;
 package Email::MIME::Header;
 # ABSTRACT: the header of a MIME message
-$Email::MIME::Header::VERSION = '1.940';
+$Email::MIME::Header::VERSION = '1.944';
 use parent 'Email::Simple::Header';
 
+use Carp ();
 use Email::MIME::Encode;
 use Encode 1.9801;
+use Module::Runtime ();
+
+our @CARP_NOT;
+
+our %header_to_class_map;
+
+{
+  my @address_list_headers = qw(from sender reply-to to cc bcc);
+  push @address_list_headers, map { "resent-$_" } @address_list_headers;
+  push @address_list_headers, map { "downgraded-$_" } @address_list_headers; # RFC 5504
+  push @address_list_headers, qw(original-from disposition-notification-to); # RFC 5703 and RFC 3798
+  $header_to_class_map{$_} = 'Email::MIME::Header::AddressList' foreach @address_list_headers;
+}
 
 #pod =head1 DESCRIPTION
 #pod
@@ -15,6 +29,7 @@ use Encode 1.9801;
 #pod
 #pod =for :list
 #pod * the C<header> method automatically decodes encoded headers if possible
+#pod * the C<header_as_obj> method returns an object representation of the header value
 #pod * the C<header_raw> method returns the raw header; (read only for now)
 #pod * stringification uses C<header_raw> rather than C<header>
 #pod
@@ -74,12 +89,46 @@ sub header_str_pairs {
   return @pairs;
 }
 
+sub header_as_obj {
+  my ($self, $name, $index, $class) = @_;
+
+  $class = $self->get_class_for_header($name) unless defined $class;
+
+  {
+    local @CARP_NOT = qw(Email::MIME);
+    local $@;
+    Carp::croak("No class for header '$name' was specified") unless defined $class;
+    Carp::croak("Cannot load package '$class' for header '$name': $@") unless eval { Module::Runtime::require_module($class) };
+    Carp::croak("Class '$class' does not have method 'from_mime_string'") unless $class->can('from_mime_string');
+  }
+
+  my @values = $self->header_raw($name, $index);
+  if (wantarray) {
+    return map { $class->from_mime_string($_) } @values;
+  } else {
+    return $class->from_mime_string(@values);
+  }
+}
+
 sub _maybe_decode {
   my ($name, $str_ref) = @_;
   $$str_ref = Email::MIME::Encode::maybe_mime_decode_header($name, $$str_ref);
   return;
 }
 
+sub get_class_for_header {
+  my ($self, $name) = @_;
+  return $header_to_class_map{lc $name};
+}
+
+sub set_class_for_header {
+  my ($self, $class, $header) = @_;
+  $header = lc $header;
+  Carp::croak("Class for header '$header' is already set") if defined $header_to_class_map{$header};
+  $header_to_class_map{$header} = $class;
+  return;
+}
+
 1;
 
 __END__
@@ -94,7 +143,7 @@ Email::MIME::Header - the header of a MIME message
 
 =head1 VERSION
 
-version 1.940
+version 1.944
 
 =head1 DESCRIPTION
 
@@ -109,6 +158,10 @@ the C<header> method automatically decodes encoded headers if possible
 
 =item *
 
+the C<header_as_obj> method returns an object representation of the header value
+
+=item *
+
 the C<header_raw> method returns the raw header; (read only for now)
 
 =item *
